from scobra.classes import db, fca, flux, fva, matrix, model, pareto 

m1 = ("sample/testmodel.xls")
m2 = ("sample/testmodel2.xls")

def CompareModel_test():
    assert model.CompareModel(m1,m1) == 
    assert model.CompareModel(m1,m2)



# coding: utf-8

# In[1]:


import timeit 
start = timeit.default_timer()
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import scobra
import get_all_functions as gaf
import re
def fresh(): 
    new = scobra.Model(os.path.join(currentdir, "sample", "Diel_Leaf_Model_cleaned.xls"))
    #new = scobra.model(n)

    #new=scobra.Model('models/Diel_Leaf_Model_cleaned.xls')

    #Maximize Sucrose output (objective)
    #vary light (scan)
    #allow CO2 and O2, even Proton and Water
    #Output maintenance ATP and NADPHox
    tx_and_biomass = []
    for i in new.Reactions(): 
        if '_tx' in i:
            tx_and_biomass.append(re.search('(.*)_tx', i).group())
        elif '_biomass' in i: 
            tx_and_biomass.append(re.search('(.*)_biomass', i).group())
    #tx_and_biomass.append(re.search('(.*)_tx', 'slkdfjd_tx').group())
    tx_and_biomass

    necessary_rxn= ['ATPase_tx', 'CO2_tx', 'Photon_tx', 'O2_tx','H2O_tx', 'H_tx','NADPHoxc_tx','NADPHoxm_tx','NADPHoxp_tx','Sucrose_tx']
    unecessary_rxn= list(set(tx_and_biomass) - set(necessary_rxn))

    #blocking _tx and _biomass reactions that are not considered
    for i in unecessary_rxn: 
        new.GetReaction(i).lower_bound = 0.0
        new.GetReaction(i).upper_bound = 0.0

    #freeing _tx and _biomass reactions that are considered
    for i in necessary_rxn: 
        if new.GetReaction(i).lower_bound == 0.0:
            new.GetReaction(i).upper_bound == 1000.0
        else: 
            new.GetReaction(i).lower_bound == 1000.0
            new.GetReaction(i).upper_bound == 1000.0

    #setting constraints
    new.SetConstraint('ATPase_tx', 90, 90)
    new.SetConstraint('NADPHoxc_tx', 10, 10)
    #new.SetConstraint('NADPHoxm_tx', 10, 10)
    #new.SetConstraint('NADPHoxp_tx', 10, 10)
    #setting constraints for sucrose entering and exiting cytoplasm
    new.SetConstraint('Sucrose_ec', -1000, 1000)

    #setting objectives
    new.SetObjective('Sucrose_tx')
    new.SetObjDirec('Min')

    #setting constraint and running scan
    #result = {}
    #for i in range(0,1000,20):
    #    new.SetConstraint('Photon_tx', i, i)
    #    new.Solve()
    #    x = new.GetSol()['Sucrose_tx']
    #    result[i] = x

    #new.Solve() 
    #new.MinFluxSolve()

    new.Solve()
    return new
new = fresh()


# In[2]:


import scobra.analysis.Scan as scan


# In[3]:


scan.ConstraintScan(new, {'NADPHoxm_tx': 10, 'NADPHoxp_tx': 10}, 0, 100, 20, MinFlux = False)


# In[4]:


scan.RatioScan(new, 'NADPHoxm_tx', 'NADPHoxp_tx', 20)


# In[ ]:


#new = fresh()
#new.ConstraintRandomMinFluxScan({'NADPHoxm_tx': 10, 'NADPHoxp_tx': 10}, 0,500, 9, 1)
#doesn't work when the 4th argument is 10 for some reason...


# In[ ]:


#new = fresh()
#new.RatioRandomMinFluxScan('NADPHoxm_tx','NADPHoxp_tx', 10, 1)


# In[5]:


new.WeightingScan({'NADPHoxm_tx': 10, 'NADPHoxp_tx': 10}, 0, 500, 10)


# In[6]:


end = timeit.default_timer() 
print(end - start)
#print('Test Completed')




import functions

function_dict = {
    "CompareModel" = functions.CompareModel_test
}

def test_x(function_name):
    """
    params function_name(str): name of functions 
    
    return 0 if successful 
    """
    return function_dict[function_name]()



# coding: utf-8

# In[19]:


import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import scobra
import timeit
start = timeit.default_timer()
model_path = os.path.join(currentdir, "sample","testmodel.xls")
n = scobra.Model(model_path)
m = scobra.model(n)

def blockPrint():
    sys.stdout = open(os.devnull, 'w')

def enablePrint():
    sys.stdout = sys.__stdout__

blockPrint()
# In[20]:


###import random
##[m.Genes()[i] for i in random.sample(range(len(m.Genes())), 5)]


# In[21]:


# Creating sample reactions
reac_sample = ['PGLUCISOM_RXN_c', 'ACONITATEHYDR_RXN_c', '2TRANSKETO_RXN_p', 'Pi_PROTON_mc', 'CYS_PROTON_vc']
met_sample = ['PROTON_v', 'SUCROSE_c', 'UBIQUINONE_mi', 'PROTON_e', 'PPI_c']
gen_sample = ['SOLYC12G099650.1', 'SOLYC10G018300.1', 'SOLYC05G008370.1', 'SOLYC02G020940.2', 'SOLYC01G089970.2']


# In[22]:


# MANIPULATING AND WRITING MODELS
m.Copy()
m.SubModel(reac_sample)
m.DuplicateModel(["_c", "_t"])
m.WriteModel(os.path.join(currentdir, "misc", "writemodel.xls"))
m.WriteModel(os.path.join(currentdir, "misc", "writemodel.sbml"))
m.WriteAttributes(os.path.join(currentdir, "misc", "writeattr"))
m.WriteNetwork(os.path.join(currentdir, "misc", "writenetwork"))


# In[23]:


### REACTIONS 
# GETTING REACTIONS
m.GetReactions(reac_sample)
m.GetReactionNames(reac_sample)
m.Isozymes()
# PRINTING REACTIONS
m.PrintReactions(reac_sample)
# ADDING AND REMOVING REACTIONS
m.AddReaction("R1", {"A": 1, "B":-1})
m.ChangeReactionStoichiometry("R1", {"A": 2, "B":-5})
m.DelReactions(["R1"])
# IMBALANCE REACTIONS
m.ImbalanceReactions()
m.CheckReactionBalance(reac_sample[0])


# In[24]:


### METABOLITES
# GETTING METABOLITES
m.GetMetabolites(met_sample)
m.GetMetaboliteNames(met_sample)
m.Metabolites()
# ADDING AND REMOVING METABOLITES
m.AddReaction("R1", {"A": 1, "B":-1})
m.AddMetabolite("D")
m.SubstituteMetabolite("A", "D")
m.DelMetabolites(["A"])
m.DelReaction("R1")
m.AddProtonsToMets({m.GetMetabolite(met_sample[1]): 2, m.GetMetabolite(met_sample[2]): 3},"PROTON_e")
m.AssignMetabolitesNeutralFormula()


# In[25]:


# SOLVING AND PRINTING SOLUTIONS
m.Solve()
m.GetSol()
m.PrintSol()
m.MinFluxSolve()
m.AdjustedMinFluxSolve()
#m.NetStoi
m.ProduceMetabolites()
m.BlockedMetabolites()


# In[26]:


# PARETO 
the_sample = [['PHOSGLYPHOS_RXN_c'], {'H_tx':3, 'F16BDEPHOS_RXN_c':2}]
m.Pareto(the_sample, "Min", 10) # every single run outputs the same result 


# In[27]:


# FLUX SUM
m.FluxSum('PROTON_m')


# In[28]:


# FLUX RANGE
m.RandomMinFlux()


# In[29]:


# GRAPHS
m.DeadEndMetabolites()
m.PeripheralMetabolites()
m.ChokepointReactions()
m.GetNeighbours('PROTON_v')
m.DegreeDist()
m.MetabolitesDegree()
m.ReactionsDegree()


# In[30]:


end = timeit.default_timer()
print(end - start)


# In[ ]:


#import random
#sample = random.sample(range(len(m.Reactions())), 3)
#sample
#the_sample = [m.Reactions()[x] for x in sample]
#the_sample = ['PHOSGLYPHOS_RXN_c', 'H_tx', 'F16BDEPHOS_RXN_c']





# coding: utf-8

# In[ ]:


import timeit 
start = timeit.default_timer()


# In[3]:


import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import scobra
model_path = os.path.join(currentdir, "sample","testmodel.xls")
n = scobra.Model(model_path)
m = scobra.model(n)

def blockPrint():
    sys.stdout = open(os.devnull, 'w')

def enablePrint():
    sys.stdout = sys.__stdout__

blockPrint()

# In[4]:


m.AllFluxRange()


# In[5]:


m.FluxRange({'O2_tx': 1})


# In[6]:


m.InternalCycles()


# In[8]:


end = timeit.default_timer()
print(end - start)





# coding: utf-8

# In[1]:


import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import scobra
import timeit
start = timeit.default_timer()
model_path = os.path.join(currentdir, "sample","testmodel.xls")
n = scobra.Model(model_path)
m = scobra.model(n)


# In[2]:


m.FCA() 


# In[3]:


end = timeit.default_timer()
end - start




import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import timeit 

def blockPrint():
    sys.stdout = open(os.devnull, 'w')

def enablePrint():
    sys.stdout = sys.__stdout__

def test(FCA=False):
	start = timeit.default_timer()
	blockPrint()
	try:
		print('executing Model')
		path = os.path.join(currentdir, "Model.py")
		command = 'python %s' % (path)
		os.system(command)
	except Exception as e:
		print(e) 
	try:
		print('executing MOMA and ROOM')
		import MOMA_and_ROOM
	except Exception as e:
		print(e) 
	try:
		print('executing FVA')
		path = os.path.join(currentdir, "FVA.py")
		command = 'python %s' % (path)
		os.system(command)
	except Exception as e:
		print(e) 
	try:
		print('executing Scan')
		import Scan
	except Exception as e:
		print(e) 
	if FCA: 
		try:
			print('executing FCA')
			import FCA
		except Exception as e:
			print(e) 
	end  = timeit.default_timer()
	time_taken = end - start
	enablePrint()
	print("Test completed in %s seconds" % (time_taken))




# coding: utf-8

# In[2]:


import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import scobra
import get_all_functions as gaf
import re

new=scobra.Model('sample/12018_0_supp_510122_p0t431.xls')
new= scobra.model(new)


# In[ ]:


new.Reactions()


# In[ ]:


for i in new.Reactions(): 
    if new.GetReaction(i).lower_bound == 0.0:
        new.GetReaction(i).upper_bound == 1000.0
    else: 
        new.GetReaction(i).lower_bound == 1000.0
        new.GetReaction(i).upper_bound == 1000.0
new.SetConstraint('O2_pc', -1, -1)
new.SetObjective('Photon_tx')
new.SetObjDirec('Max')
new.Solve()


# In[ ]:


len(new.GetSol())


# In[ ]:


#Maximize Sucrose output (objective)
#vary light (scan)
#allow CO2 and O2, even Proton and Water
#Output maintenance ATP and NADPHox
tx_and_biomass = []
for i in new.Reactions(): 
    if '_tx' in i:
        tx_and_biomass.append(re.search('(.*)_tx', i).group())
    elif '_biomass' in i: 
        tx_and_biomass.append(re.search('(.*)_biomass', i).group())
#tx_and_biomass.append(re.search('(.*)_tx', 'slkdfjd_tx').group())
tx_and_biomass

necessary_rxn= ['CO2_tx', 'Photon_tx', 'O2_tx','H2O_tx','NADPHoxc_tx','NADPHoxm_tx','NADPHoxp_tx','Starch_tx']
unecessary_rxn= list(set(tx_and_biomass) - set(necessary_rxn))

#blocking _tx and _biomass reactions that are not considered
for i in unecessary_rxn: 
    new.GetReaction(i).lower_bound = 0.0
    new.GetReaction(i).upper_bound = 0.0

#freeing _tx and _biomass reactions that are considered
for i in necessary_rxn: 
    if new.GetReaction(i).lower_bound == 0.0:
        new.GetReaction(i).upper_bound == 1000.0
    else: 
        new.GetReaction(i).lower_bound == 1000.0
        new.GetReaction(i).upper_bound == 1000.0

#setting constraints
#new.SetConstraint('ATPase_tx', 90, 90)
#new.SetConstraint('NADPHoxc_tx', 10, 10)
#new.SetConstraint('NADPHoxm_tx', 10, 10)
#new.SetConstraint('NADPHoxp_tx', 10, 10)
#setting constraints for sucrose entering and exiting cytoplasm
new.SetConstraint('Starch_tx', -1, -1)

#setting objectives
new.SetObjective('Photon_tx')
new.SetObjDirec('Min')
#new.SetFixedFlux({'unlProtHYPO_c': 0})

#running scans
#new.MinFluxSolve() 
new.Solve()


# In[2]:


start = timeit.timeit()
new.MinFluxSolve()
end = timeit.timeit()
time_taken = end - start


# In[4]:


tx_and_biomass = []
for i in new.Reactions(): 
    if '_tx' in i:
        tx_and_biomass.append(re.search('(.*)_tx', i).group())
new.PrintReactions(tx_and_biomass)


# In[3]:


new.GenesToSubsystemsAssociations


# In[27]:


len(new.GetSol())


# In[29]:


len(new.Reactions())


# In[6]:


#new.WriteModel('/home/kristoforus/Desktop/model.xls')


# In[31]:


lst_submodel = [new.GetReaction(x) for x in new.GetSol()]
testm = new.SubModel(lst_submodel)

print( [ i for i in testm.Reactions() if 'ATP' in i])


# In[32]:


len(testm.Reactions())


# In[33]:


testm.SetObjective('Photon_tx')
testm.SetObjDirec('Min')


# In[35]:


testm.Solve()
testm.GetSol()
len(testm.GetSol())


# In[37]:


testm.WriteModel('/home/kristoforus/Desktop/testmodel.xls')





# coding: utf-8

# In[27]:


import timeit 
start = timeit.default_timer()
import os, sys, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
gparentdir = os.path.dirname(os.path.dirname(currentdir))
sys.path.insert(0, gparentdir)
import scobra
import get_all_functions as gaf
import re
new = scobra.Model(os.path.join(currentdir, "sample", "Diel_Leaf_Model_cleaned.xls"))
#new = scobra.model(n)

#new=scobra.Model('models/Diel_Leaf_Model_cleaned.xls')

#Maximize Sucrose output (objective)
#vary light (scan)
#allow CO2 and O2, even Proton and Water
#Output maintenance ATP and NADPHox
tx_and_biomass = []
for i in new.Reactions(): 
    if '_tx' in i:
        tx_and_biomass.append(re.search('(.*)_tx', i).group())
    elif '_biomass' in i: 
        tx_and_biomass.append(re.search('(.*)_biomass', i).group())
#tx_and_biomass.append(re.search('(.*)_tx', 'slkdfjd_tx').group())
tx_and_biomass

necessary_rxn= ['ATPase_tx', 'CO2_tx', 'Photon_tx', 'O2_tx','H2O_tx', 'H_tx','NADPHoxc_tx','NADPHoxm_tx','NADPHoxp_tx','Sucrose_tx']
unecessary_rxn= list(set(tx_and_biomass) - set(necessary_rxn))

#blocking _tx and _biomass reactions that are not considered
for i in unecessary_rxn: 
    new.GetReaction(i).lower_bound = 0.0
    new.GetReaction(i).upper_bound = 0.0

#freeing _tx and _biomass reactions that are considered
for i in necessary_rxn: 
    if new.GetReaction(i).lower_bound == 0.0:
        new.GetReaction(i).upper_bound == 1000.0
    else: 
        new.GetReaction(i).lower_bound == 1000.0
        new.GetReaction(i).upper_bound == 1000.0

#setting constraints
new.SetConstraint('ATPase_tx', 90, 90)
new.SetConstraint('NADPHoxc_tx', 10, 10)
new.SetConstraint('NADPHoxm_tx', 10, 10)
new.SetConstraint('NADPHoxp_tx', 10, 10)
#setting constraints for sucrose entering and exiting cytoplasm
new.SetConstraint('Sucrose_ec', -1000, 1000)

#setting objectives
new.SetObjective('Sucrose_tx')
new.SetObjDirec('Min')

#setting constraint and running scan
"""
result = {}
for i in range(0,1000,20):
    new.SetConstraint('Photon_tx', i, i)
    new.Solve()
    x = new.GetSol()['Sucrose_tx']
    result[i] = x
"""
#new.Solve() 
#new.MinFluxSolve()

new.Solve()


# In[28]:


refflux = {'1TRANSKETO_RXN_p': 31.979030144167627,
 '1_PERIOD_10_PERIOD_2_PERIOD_2_RXN_mi': 28.12581913499394,
 '1_PERIOD_18_PERIOD_1_PERIOD_2_RXN_p': 250.00000000000009,
 '2KG_ISOCITRATE_mc': 9.999999999999966,
 '2TRANSKETO_RXN_p': -31.979030144167627,
 'ADENYL_KIN_RXN_c': -42.188728702489634,
 'ADENYL_KIN_RXN_m': 42.18872870248983,
 'ATP_ADP_Pi_pc': 13.617300131062754}


# In[29]:


import scobra.analysis.MOMA as moma
moma.LinearMOMA(new, refflux)
moma.MOMA(new, refflux)


# In[30]:


new.ROOM(refflux)


# In[31]:


start = timeit.default_timer()
new.MinReactionsSolve()
end = timeit.default_timer()
end - start


# In[32]:


end = timeit.default_timer()
print(end - start)
#print('Test Completed')




import setuptools
import os
thelibFolder = os.path.dirname(os.path.realpath(__file__))
requirementPath = thelibFolder + '/requirements.txt'
install_requires = [] # Better to write it in requirements.txt
if os.path.isfile(requirementPath):
    with open(requirementPath) as f:
        install_requires = f.read().splitlines()

with open ("README.md","r") as fh:
    long_description=fh.read()

setuptools.setup(
    name="scobra",
    version="0.0.2",
    author="Maurice Cheung",
    author_email="maurice.cheung@yale-nus.edu.sg",
    description="Supplementary package for COBRApy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mauriceccy/scobra",
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)




# coding: utf-8

# In[10]:


import re 


# In[11]:




p = '\s*def (.*)\(.*'
p_easy = '\s*def (.*)\(self*'
p_self = '\s*def (.*)\(self\).*'


# In[12]:


def get_function_names(file): 
	thelist = []
	for i, line in file:
		if re.match(p, line) != None: 
			thelist.append(re.match(p, line).group(1))
		else: 
			pass
	print thelist
	return thelist

def get_function_names_easy(file): 
	thelist = []
	for i, line in file:
		if re.match(p_easy, line) != None: 
			thelist.append(re.match(p_easy, line).group(1))
		else: 
			pass
	print thelist
	return thelist

def get_functions_self_only(file): 
	self_only = []
	more = []
	for i, line in file: 
		if re.match(p_self, line) != None: 
			self_only.append(re.match(p_self,line).group(1))
		elif re.match(p, line) != None: 
			more.append(re.match(p, line).group(1))
		else: 
			pass
	return self_only, more
	

